local S = minetest.get_translator("lzr_infobooks")
local FS = function(...) return minetest.formspec_escape(S(...)) end
local F = minetest.formspec_escape

lzr_infobooks = {}

local registered_books = {}
local registered_bookshelves = {}
local current_bookshelf = nil
local current_bookshelf_selection_index = nil

lzr_infobooks.register_book = function(book_id, book_def)
	registered_books[book_id] = book_def
end
lzr_infobooks.register_bookshelf = function(bookshelf_id, bookshelf_def)
	registered_bookshelves[bookshelf_id] = bookshelf_def
end

lzr_infobooks.open_book_page = function(player_name, book_id, page_num)
	local book = registered_books[book_id]
	local max_page = #book.pages
	if page_num > max_page then
		page_num = max_page
	end
	local page = book.pages[page_num]
	local bg = "lzr_infobooks_ui_book_bg.png"
	if book.color then
		bg = bg .. "^(lzr_infobooks_ui_book_bg_overlay.png^[multiply:"..book.color..")"
	end
	local form = "formspec_version[7]size[13,8.53125]"..
		"background[0,0;13,8.53125;"..bg..";true]"..
		"label[0.65,0.52;"..F(page.title).."]"..
		"label[7.15,0.52;"..F(page.title).."]"
	if page.image or page.item_image then
		local itype
		if page.image then
			itype = "image"
		else
			itype = "item_image"
		end
		if type(page[itype]) == "table" then
			local y = 1.17
			for i=1, #page[itype] do
				form = form .. itype.."[1.885,"..y..";2.6,2.6;"..page[itype][i].."]"
				y = y + 2.795
			end
		else
			form = form .. itype.."[1.885,2.275;2.6,2.6;"..page[itype].."]"
		end
	end
	form = form ..
		"textarea[7.15,1.3;5.2,5.72;;;"..F(page.text).."]"
	if page_num > 1 then
		form = form .. "style[prev_page;sound=lzr_infobooks_turn_page]" ..
		"button[0.65,7.15;1.3,1.04;prev_page;"..FS("<").."]"..
		"tooltip[prev_page;"..FS("Previous page").."]"
	end
	if page_num < max_page then
		form = form .. "style[next_page;sound=lzr_infobooks_turn_page]" ..
		"button[11.05,7.15;1.3,1.04;next_page;"..FS(">").."]"..
		"tooltip[next_page;"..FS("Next page").."]"
	end
	minetest.show_formspec(player_name, "lzr_infobooks:book:"..book_id..":"..page_num, form)
end

lzr_infobooks.open_book = function(player_name, book_id)
	minetest.sound_play({name = "lzr_infobooks_open_book", gain = 0.4}, {to_player=player_name}, true)
	lzr_infobooks.open_book_page(player_name, book_id, 1)
end

lzr_infobooks.open_bookshelf = function(player_name, bookshelf_id)
	local books = registered_bookshelves[bookshelf_id].books

	local form = "formspec_version[7]size[6,5]"..
		"label[0.5,0.5;"..FS("Select a book:").."]"..
		"style[read;sound=]"..
		"button[1.5,3.5;3,1;read;"..FS("Read").."]"..
		"tablecolumns[text]"..
		"table[0.5,0.8;5,2.5;booklist;"
	for b=1, #books do
		local book = registered_books[books[b]]
		form = form .. F(book.title)
		if b < #books then
			form = form .. ","
		end
	end
	local idx = current_bookshelf_selection_index or 1
	form = form .. ";"..idx.."]"
	current_bookshelf = bookshelf_id
	current_bookshelf_selection_index = 1
	minetest.show_formspec(player_name, "lzr_infobooks:bookshelf:"..bookshelf_id, form)
end

minetest.register_on_player_receive_fields(function(player, formname, fields)
	if string.sub(formname, 1, 13) ~= "lzr_infobooks" then
		current_bookshelf = nil
		current_bookshelf_selection_index = nil
		return
	end
	local split = string.split(formname, ":")
	if not split or not split[2] or not split[3] then
		return
	end
	local btype = split[2]
	if btype == "book" then
		if fields.quit then
			minetest.sound_play({name = "lzr_infobooks_close_book", gain = 0.4}, {to_player=player:get_player_name()}, true)
			if current_bookshelf then
				lzr_infobooks.open_bookshelf(player:get_player_name(), current_bookshelf)
			end
			return
		end
		local book_id = split[3]
		local page_num = tonumber(split[4])
		if not page_num then
			return
		end

		if fields.next_page then
			page_num = page_num + 1
			lzr_infobooks.open_book_page(player:get_player_name(), book_id, page_num)
		elseif fields.prev_page then
			page_num = math.max(1, page_num - 1)
			lzr_infobooks.open_book_page(player:get_player_name(), book_id, page_num)
		end
	elseif btype == "bookshelf" then
		if fields.quit then
			current_bookshelf = nil
			current_bookshelf_selection_index = nil
			return
		end
		local bookshelf_id = split[3]
		if fields.booklist then
			local expl = minetest.explode_table_event(fields.booklist)
			if expl.type == "DCL" then
				local bookshelf = registered_bookshelves[bookshelf_id]
				local book_id = bookshelf.books[expl.row]
				lzr_infobooks.open_book(player:get_player_name(), book_id)
			elseif expl.type == "CHG" then
				current_bookshelf_selection_index = expl.row
			elseif expl.type == "INV" then
				current_bookshelf_selection_index = nil
			end
			return
		end
		if fields.read then
			if current_bookshelf and current_bookshelf_selection_index then
				local bookshelf = registered_bookshelves[current_bookshelf]
				local book_id = bookshelf.books[current_bookshelf_selection_index]
				lzr_infobooks.open_book(player:get_player_name(), book_id)
			end
		end
	end
end)

-- This help category is for blocks that interact with lasers and
-- have an active and inactive state.
local laser_blocks = {
	{ "lzr_laser:emitter_red_takable", nil, S("Emitter"), S("This block emits a laser when turned on. Emitters come in different colors.") },
	{ "lzr_laser:detector_takable", nil, S("Detector (colorless)"), S("This block has a hole and will be activated when a laser goes into it. It may then trigger something.") },
	{ "lzr_laser:detector_red_takable", nil, S("Detector (colored)"), S("Like the colorless detector, it will be activated when a laser goes into the hole. But colored detectors require the laser to be of the same color.") },
	{ "lzr_laser:mirror_takable", nil, nil, S("When a laser hits the mirror, it will make a turn.") },
	{ "lzr_laser:double_mirror_00_takable", "lzr_laser:double_mirror_10_takable", nil, S("The double mirror has a reflective surface on two sides, so it can deflect two lasers at once.") },
	{ "lzr_laser:transmissive_mirror_00_takable", "lzr_laser:transmissive_mirror_01_takable", nil, S("When a laser hits the angled part of a beam splitter, it splits it into two. One part of the laser goes straight through while the other part will be deflected.") .."\n" .. S("Shooting a laser at the backside of this block won’t to anything.") },
	{ "lzr_laser:crystal_takable", "lzr_laser:crystal_takable_on_1", nil, S("When a laser goes into a crystal, it will spread in all directions.") },
	{ "lzr_laser:mixer_takable", "lzr_laser:mixer_takable_on_1", nil, S("The mixer has two inputs to the left and right and one output at the front. When two lasers go into the input, a laser with their colors mixed will go out of the output.").."\n"..S("When only one input is used, the same laser will go out of the output unchanged.") },
	{ "lzr_laser:skull_shy_takable", nil, nil, S("Skulls have holes on each side and lasers fired at it will go straight through.").."\n"..S("This skull is called “shy” because it’s afraid of lasers. When any laser touches it, it will become ghostly and cannot be touched or walked on.") },
	{ "lzr_laser:skull_cursed_takable", nil, nil, S("Skulls have holes on each side and lasers fired at it will go straight through.").."\n"..S("The cursed skulls have lost their physical form and cannot be touched or walked on. They can be brought back into the physical realm by firing a laser at it.") },
	{ "lzr_laser:bomb_takable", nil, nil, S("The bomb will blow up cracked stone and other fragile blocks in a 3×3×3 radius when its fuse is ignited by a laser or fire.") },
	{ "lzr_laser:barricade", "lzr_laser:barricade_on", nil, S("Barricades are flammable and burn up when hit by a laser. They also ignite all flammable blocks and fuses next to them.") },
	{ "lzr_laser:hollow_barrel_takable", nil, nil, S("A simple obstacle with an opening where a laser can go through.") },
}

local lb_pages = {}
for i=1, #laser_blocks do
	local basename = laser_blocks[i][1]
	local nodedef = minetest.registered_nodes[basename]
	if not nodedef then
		error("Missing nodedef for "..basename)
	end
	local title = laser_blocks[i][3] or nodedef.description or "???"
	local text = laser_blocks[i][4] or nodedef._tt_help or "???"
	local item_image = { basename, laser_blocks[i][2] or basename.."_on" }
	table.insert(lb_pages, { title = title, text = text, item_image = item_image })
end

lzr_infobooks.register_book("laser_blocks", {
	title = S("Laser Blocks"),
	color = "#8a0505",
	pages = lb_pages,
})

-- This help category is for all noteworthy blocks (relevant for gameplay)
-- that don't directly interact with lasers or don't have an active/inactive state.
local other_blocks = {
	{ "lzr_core:stone", "lzr_core:wood", S("Solid Block"), S("Most blocks are solid. This includes stone, wood, tree trunks, furniture, dirt, sand, and more. Solid blocks cannot be destroyed, picked up, rotated or moved.") },
	{ "lzr_core:stone_cracked", "lzr_core:cave_stone_cracked", S("Cracked Stone"), S("Cracked stone will be destroyed by explosions.") },
	{ "lzr_laser:crate_takable", nil, nil, S("Unlike the darker heavy crate, a light crate can be picked up and carried around. You can use it to reach higher places.") },
	{ "lzr_laser:crate_old", "lzr_decor:barrel_old", S("Old Wood"), S("Old crates and barrels fall into pieces at the slightest touch. Explosions also destroy them.") },
	{ "lzr_decor:iron_lightbox_off", "lzr_decor:iron_lightbox", S("Light Box"), S("Light boxes create light. They may be turned on or off by detectors.") },
	{ "lzr_decor:cobweb", nil, nil, S("Cobwebs make it impossible to jump while inside them. You also can’t touch anything behind a cobweb.").."\n"..S("Cobwebs can be destroyed with lasers.") },
}
local ob_pages = {}
for i=1, #other_blocks do
	local basename = other_blocks[i][1]
	local basename2 = other_blocks[i][2]
	local nodedef = minetest.registered_nodes[basename]
	if not nodedef then
		error("Missing nodedef for "..basename)
	end
	local nodedef2 = minetest.registered_nodes[basename2]
	if basename2 and not nodedef2 then
		error("Missing nodedef for "..basename2)
	end
	local title = other_blocks[i][3] or nodedef.description or "???"
	local text = other_blocks[i][4] or nodedef._tt_help or "???"
	local item_image
	if basename2 then
		item_image = { basename, basename2 }
	else
		item_image = basename
	end
	table.insert(ob_pages, { title = title, text = text, item_image = item_image })
end

lzr_infobooks.register_book("other_blocks", {
	title = S("Other Blocks"),
	color = "#8a4505",
	pages = ob_pages,
})

lzr_infobooks.register_book("treasures", {
	title = S("Treasures"),
	color = "#8a4f05",
	pages = {
		{
			title = S("Gold Block"),
			text = S("The goal of every level is to find all gold blocks."),
			item_image = "lzr_treasure:gold_block",
		},
		{
			title = S("Treasure Chests"),
			text = S("You can find treasure chests all over the world. You can find a gold block inside.").."\n"..
				S("To open an unlocked chest, just punch it."),
			item_image = {"lzr_treasure:chest_wood_unlocked", "lzr_treasure:chest_wood_open_gold"},
		},
		{
			title = S("Breaking the lock"),
			text = S("Many chests are locked. To break the lock, you need to solve a laser puzzle."),
			item_image = "lzr_treasure:chest_wood_locked",
		},
		{
			title = S("Wooden Chests and Dark Chests"),
			text = S("There two types of chests:").."\n\n"..
				S("Wooden Chest: When the lock has been broken, it stays broken.").."\n"..
				S("Dark Chest: The lock can be broken, but it will regenerate when the laser puzzle is no longer solved."),
			item_image = { "lzr_treasure:chest_wood_locked", "lzr_treasure:chest_dark_locked"},
		},
	},
})

lzr_infobooks.register_book("get_rich", {
	title = S("How to get rich"),
	color = "#c7b000",
	pages =	{{
		title = S("How to get rich"),
		text = S("Step 1: Go to place with hidden treasure").."\n"..
			S("Step 2: Crack the laser security mechanism (if any)").."\n"..
			S("Step 3: Loot treasure chests for gold").."\n"..
			S("Step 4: Repeat steps 1 until 3 until you’re rich"),
		item_image = "lzr_treasure:gold_block",
	}},
})

-- TODO: Use this book somewhere
lzr_infobooks.register_book("pirate_code", {
	title = S("The Pirate’s Code"),
	color = "#c7b000",
	pages =	{{
		title = S("The Pirate’s Code"),
		text = S("Steal from the rich and give it to ourselves"),
		item_image = "lzr_treasure:gold_block",
	}},
})

lzr_infobooks.register_book("screws", {
	title = S("Screws"),
	color = "#afafaf",
	pages = {
		{
			title = S("Screws"),
			text = S("While you can pick up some blocks, some blocks have screws in their corners. Blocks with screws cannot be picked up."),
			image = "[inventorycube{lzr_laser_fixed.png{lzr_laser_fixed.png{lzr_laser_fixed.png",
		},
		{
			title = S("Screw types"),
			text = S("There are two types of screws:").."\n\n"..
				S("Copper screws (brown): Block can be rotated.").."\n"..
				S("Iron screws (gray): Block cannot be rotated."),
			image = {
				"[inventorycube{lzr_laser_fixed_soft.png{lzr_laser_fixed_soft.png{lzr_laser_fixed_soft.png",
				"[inventorycube{lzr_laser_fixed.png{lzr_laser_fixed.png{lzr_laser_fixed.png",
			},
		},
	},
})

local color_page_texture = "lzr_infobooks_colors.png"
local colormix_page_texture = "lzr_infobooks_colormix.png"
local colormix2_page_texture = "lzr_infobooks_colormix2.png"
do
	local colors = {
		{ "red", lzr_laser.LASER_COLORS[lzr_globals.COLOR_RED] },
		{ "green", lzr_laser.LASER_COLORS[lzr_globals.COLOR_GREEN] },
		{ "blue", lzr_laser.LASER_COLORS[lzr_globals.COLOR_BLUE] },
		{ "yellow", lzr_laser.LASER_COLORS[lzr_globals.COLOR_YELLOW] },
		{ "cyan", lzr_laser.LASER_COLORS[lzr_globals.COLOR_CYAN] },
		{ "magenta", lzr_laser.LASER_COLORS[lzr_globals.COLOR_MAGENTA] },
		{ "white", lzr_laser.LASER_COLORS[lzr_globals.COLOR_WHITE] },
	}
	for c=1, #colors do
		color_page_texture = color_page_texture .. "^(lzr_infobooks_colors_"..colors[c][1]..".png^[multiply:#"..colors[c][2]..")"
		if colors[c][1] ~= "white" then
			colormix_page_texture = colormix_page_texture .. "^(lzr_infobooks_colormix_"..colors[c][1]..".png^[multiply:#"..colors[c][2]..")"
		end
		if colors[c][1] ~= "yellow" and colors[c][1] ~= "magenta" and colors[c][1] ~= "cyan" then
			colormix2_page_texture = colormix2_page_texture .. "^(lzr_infobooks_colormix2_"..colors[c][1]..".png^[multiply:#"..colors[c][2]..")"
		end
	end
end

lzr_infobooks.register_book("laser_color", {
	title = S("Laser Colors"),
	color = "#00b52e",
	pages = {
		{
			title = S("Laser Colors"),
			text = S("Lasers come in 7 different colors: Red, green, blue, yellow, cyan, magenta and white."),
			image = color_page_texture,
		},
		{
			title = S("Color Mixing"),
			text = S("When two lasers overlap or get mixed in a mixer, their colors will be combined.").."\n\n"..
				S("Red + Green → Yellow").."\n"..
				S("Red + Blue → Magenta").."\n"..
				S("Green + Blue → Cyan").."\n"..
				S("Red + Green + Blue → White"),
			image = { colormix_page_texture, colormix2_page_texture },
		},
	},
})

lzr_infobooks.register_book("hook", {
	title = S("Hook"),
	color = "#24948f",
	pages = {
		{
			title = S("Rotating Hook"),
			text = S("The rotating hook allows you to rotate blocks. Not all blocks can be rotated.").."\n\n"..
				S("Use the [Place] or [Punch] key to rotate.").."\n"..
				S("[Place]: Rotate the block clockwise.").."\n"..
				S("[Punch]: Rotate the block towards the edge you touched.").."\n"..
				S("Holding down [Sneak] will rotate in the reverse direction."),
			image = "lzr_hook_hook.png",
		},
	},
})

lzr_infobooks.register_bookshelf("how_to_play", {
	books = {
		"get_rich",
		"treasures",
		"laser_blocks",
		"other_blocks",
		"laser_color",
		"hook",
		"screws",
	},
})

