local mirror_out = {
	[0] = {1,0,0},
	[1] = {0,0,-1},
	[2] = {-1,0,0},
	[3] = {0,0,1},

	[4] = {1,0,0},
	[5] = {0,1,0},
	[6] = {-1,0,0},
	[7] = {0,-1,0},

	[8] = {1,0,0},
	[9] = {0,-1,0},
	[10] = {-1,0,0},
	[11] = {0,1,0},

	[12] = {0,-1,0},
	[13] = {0,0,-1},
	[14] = {0,1,0},
	[15] = {0,0,1},

	[16] = {0,1,0},
	[17] = {0,0,-1},
	[18] = {0,-1,0},
	[19] = {0,0,1},

	[20] = {-1,0,0},
	[21] = {0,0,-1},
	[22] = {1,0,0},
	[23] = {0,0,1},
}

lzr_laser.get_front_dir = function(param2)
	if not param2 then
		return
	end
	local dir_input = minetest.facedir_to_dir(param2)
	if not dir_input then
		return
	end
	local v = vector.new(dir_input[1], dir_input[2], dir_input[3])
	dir_input = vector.multiply(v, -1)
	return dir_input
end

lzr_laser.get_top_dir = function(param2)
	if not param2 then
		return
	end
	if (param2 >= 0 and param2 <= 3) then
		return vector.new(0, 1, 0)
	elseif (param2 >= 4 and param2 <= 7) then
		return vector.new(0, 0, 1)
	elseif (param2 >= 8 and param2 <= 11) then
		return vector.new(0, 0, -1)
	elseif (param2 >= 12 and param2 <= 15) then
		return vector.new(1, 0, 0)
	elseif (param2 >= 16 and param2 <= 19) then
		return vector.new(-1, 0, 0)
	elseif (param2 >= 20 and param2 <= 23) then
		return vector.new(0, -1, 0)
	else
		return vector.new(0, 1, 0)
	end

end

lzr_laser.get_barrel_axis = function(param2)
	if not param2 then
		return
	end
	if (param2 >= 0 and param2 <= 3) or (param2 >= 20 and param2 <= 23) then
		return "y"
	elseif (param2 >= 4 and param2 <= 11) then
		return "z"
	elseif (param2 >= 12 and param2 <= 19) then
		return "x"
	else
		return "y"
	end
end

lzr_laser.get_pane_axis = function(param2)
	if not param2 then
		return
	end
	local dir = minetest.facedir_to_dir(param2)
	if dir.x ~= 0 then
		return "x"
	elseif dir.y ~= 0 then
		return "y"
	elseif dir.z ~= 0 then
		return "z"
	end
end

lzr_laser.check_front = function(node_pos, laser_dir)
	local node = minetest.get_node(node_pos)
	local node_dir_in = lzr_laser.get_front_dir(node.param2)
	if not node_dir_in then
		return false
	end
	local reverse_laser_dir = vector.multiply(laser_dir, -1)
	if vector.equals(reverse_laser_dir, node_dir_in) then
		return true
	end
	return false
end

-- Check if the given laser color would activate the given
-- detector node name.
lzr_laser.check_detector_color = function(nodename, laser_color)
	local detector_group = minetest.get_item_group(nodename, "detector")
	if detector_group == 0 then
		return false
	end
	local detector_color = minetest.get_item_group(nodename, "detector_color")
	-- There is no laser: Fail
	if laser_color == 0 then
		return false
	-- Detector does not care about color: Success
	elseif detector_color == 0 then
		return true
	-- If detector *does* care about color, it must be an exact match
	elseif laser_color == detector_color then
		return true
	end
	return false
end

-- Check if laser goes into detector (ignoring color)
lzr_laser.check_detector_input = function(detector_pos, nodename, laser_dir)
	local detector_group = minetest.get_item_group(nodename, "detector")
	if detector_group == 0 then
		return false
	end
	-- Check if laser goes into hole
	if lzr_laser.check_front(detector_pos, laser_dir) then
		return true
	end
	return false
end

lzr_laser.get_mirror_dirs = function(param2)
	local dir_input = minetest.facedir_to_dir(param2)
	if not dir_input then
		return
	end
	dir_input = vector.multiply(dir_input, -1)
	local dir_output = vector.new(unpack(mirror_out[param2]))

	return dir_input, dir_output
end

lzr_laser.get_double_mirror_dirs = function(param2)
	local dir_input = minetest.facedir_to_dir(param2)
	if not dir_input then
		return
	end
	local dir_input_front = vector.multiply(dir_input, -1)
	local dir_output_front = vector.new(unpack(mirror_out[param2]))

	local dir_input_back = vector.new(dir_input.x, dir_input.y, dir_input.z)
	local dir_output_back = vector.multiply(vector.new(unpack(mirror_out[param2])), -1)

	return dir_input_front, dir_output_front, dir_input_back, dir_output_back
end

-- Mirror a laser that touches a mirror with a laser coming towards
-- the mirror with the laser_dir direction (direction vector).
-- * nodename: Name of mirror node
-- * param2: param2 of mirror node
-- * laser_dir: Direction of incoming laser
-- Returns <output direction>, <mirror ingoing>, <mirror backside>, where:
-- * <output direction>: Direction of the outgoing mirror or false if none
-- * <mirror ingoing>: true if the laser has hit the 'ingoing'
--   side of the mirror or false if not. This is because on any mirror,
--   the laser can come from two directions, the true/false distinguishes
--   which direction it was.
-- * <mirror backside>: true if the laser hit the backside of the mirror
--   (double mirror only) or false if not. nil if laser was not mirrored at all
--
-- 	1) If the laser can be mirrored, then <output direction> is the direction of
-- 	the mirrored laser and <mirror output> denotes on of the 2 possible
-- 	sides the laser came from: true if it came in the front side, false if
-- 	it came from the angled side. <mirror output> is useful for the
-- 	beam splitter (transmissive mirror).
-- 	2) If the laser cannot be mirrored <mirror output> is false and
-- 	<mirror output side> is nil
lzr_laser.get_mirrored_laser_dir = function(nodename, param2, laser_dir)
	local mirror_group = minetest.get_item_group(nodename, "mirror")
	local mirror_transmissive_group = minetest.get_item_group(nodename, "transmissive_mirror")
	local mirror_double_group = minetest.get_item_group(nodename, "double_mirror")
	if mirror_group == 0 and mirror_transmissive_group == 0 and mirror_double_group == 0 then
		return false
	end
	local reverse_laser_dir = vector.multiply(laser_dir, -1)
	if mirror_double_group ~= 0 then
		local dir_front_in, dir_front_out, dir_back_in, dir_back_out = lzr_laser.get_double_mirror_dirs(param2)
		if not dir_front_in then
			return false
		end
		if vector.equals(reverse_laser_dir, dir_front_in) then
			return dir_front_out, true, false
		elseif vector.equals(reverse_laser_dir, dir_front_out) then
			return dir_front_in, false, false
		elseif vector.equals(reverse_laser_dir, dir_back_in) then
			return dir_back_out, true, true
		elseif vector.equals(reverse_laser_dir, dir_back_out) then
			return dir_back_in, true, true
		end
	else
		local reverse_laser_dir = vector.multiply(laser_dir, -1)
		local mirror_dir_in, mirror_dir_out = lzr_laser.get_mirror_dirs(param2)
		if not mirror_dir_in then
			return false
		end
		if vector.equals(reverse_laser_dir, mirror_dir_in) then
			return mirror_dir_out, true, false
		elseif vector.equals(reverse_laser_dir, mirror_dir_out) then
			return mirror_dir_in, false, false
		end
	end
	return false
end

lzr_laser.get_mixer_input_dirs = function(param2)
	local front_dir = lzr_laser.get_front_dir(param2)
	local _, input_r = lzr_laser.get_mirror_dirs(param2)
	local input_l = vector.multiply(input_r, -1)
	return input_l, input_r
end

-- Template function for `after_place_node` callback for
-- trigger nodes.
lzr_laser.trigger_after_place_node = function(pos, placer, itemstack, pointed_thing)
	local state = lzr_gamestate.get_state()
	if state == lzr_gamestate.EDITOR then
		-- Save trigger in trigger list for the editor
		local minpos, maxpos = lzr_world.get_level_bounds()
		if vector.in_area(pos, minpos, maxpos) then
			local trigger_id = lzr_triggers.add_trigger(pos)
			if trigger_id then
				local meta = minetest.get_meta(pos)
				meta:set_string("trigger_id", trigger_id)
			end
		end
		return
	elseif state == lzr_gamestate.LEVEL then
		local minpos, maxpos = lzr_world.get_level_bounds()
		if not vector.in_area(pos, minpos, maxpos) then
			minetest.log("warning", "[lzr_laser] Placed a trigger node out of level bounds at: "..minetest.pos_to_string(pos))
			return
		end
		-- Copy item's trigger_id into node
		local imeta = itemstack:get_meta()
		local nmeta = minetest.get_meta(pos)
		local trigger_id = imeta:get_string("trigger_id")
		nmeta:set_string("trigger_id", trigger_id)

		-- Update trigger location
		lzr_triggers.set_trigger_location(trigger_id, pos)
	end
end

-- Template function for `after_dig_node` callback for
-- trigger nodes.
lzr_laser.trigger_after_dig_node = function(pos, oldnode, oldmeta, drops)
	if not oldmeta and not oldmeta and not oldmeta.fields.trigger_id then
		return
	end
	local trigger_id = oldmeta.fields.trigger_id
	if lzr_gamestate.get_state() ~= lzr_gamestate.LEVEL then
		lzr_triggers.remove_trigger(trigger_id)
		return
	end
end

